@@ -19,6 +19,7 @@ import numpy as np
 import bioread.file_revisions as rev
 from bioread import headers as bh
 from bioread.headers import GraphHeader, ChannelHeader, ChannelDTypeHeader
+from bioread.headers import UnknownPaddingHeader
 from bioread.headers import ForeignHeader, MainCompressionHeader
 from bioread.headers import ChannelCompressionHeader
 from bioread.headers import PostMarkerHeader, V2JournalHeader, V4JournalHeader
@@ -115,11 +116,23 @@ class Reader(object):
         self.graph_header = self.__single_header(0, GraphHeader)
         channel_count = self.graph_header.channel_count
 
-        ch_start = self.graph_header.effective_len_bytes
+        pad_start = self.graph_header.effective_len_bytes
+        pad_headers = self.__multi_headers(
+            self.graph_header.expected_padding_headers,
+            pad_start,
+            UnknownPaddingHeader)
+        ch_start = pad_start + sum(
+            [ph.effective_len_bytes for ph in pad_headers])
+        pad_header = self.__single_header(ch_start, UnknownPaddingHeader)
+        # if pad_header.effective_len_bytes == 40:
+        #     ch_start = 0
         self.channel_headers = self.__multi_headers(channel_count,
                                                     ch_start, ChannelHeader)
         ch_len = self.channel_headers[0].effective_len_bytes
 
+        for i, ch in enumerate(self.channel_headers):
+            logger.debug("Channel header %s: %s" % (i, ch.data))
+
         fh_start = ch_start + len(self.channel_headers)*ch_len
         self.foreign_header = self.__single_header(fh_start, ForeignHeader)
 
@@ -127,6 +140,10 @@ class Reader(object):
         self.channel_dtype_headers = self.__multi_headers(
             channel_count, cdh_start, ChannelDTypeHeader)
         cdh_len = self.channel_dtype_headers[0].effective_len_bytes
+        for i, cdt in enumerate(self.channel_dtype_headers):
+            logger.debug("Channel %s: type_code: %s, offset: %s" % (
+                i, cdt.type_code, cdt.offset
+            ))
 
         self.data_start_offset = (cdh_start + (cdh_len * channel_count))
         logger.debug("Computed data start offset: %s" % self.data_start_offset)
@@ -334,6 +351,8 @@ class Reader(object):
         self.file_revision = bp[0]
         # Guess at file encoding -- I think that everything before acq4 is
         # in latin1 and everything newer is utf-8
+        logger.debug("File revision: %s" % self.file_revision)
+        logger.debug("Byte order: %s" % self.byte_order_char)
         if self.file_revision < rev.V_400B:
             self.encoding = 'latin1'
         else:
